;;;
;;; auctex-config.el - this is the configuration file for package auctex
;;;
;;; Copyright (C) 1998, 1999, 2000 by Janek Schwarz
;;;
;;; Written by: Janek Schwarz <schwarz@informatik.hu-berlin.de>
;;;
;;; Time-stamp: <2000-03-29 21:46:37 schwarz>
;;;

(require 'config-general)

(require 'tex-site)
(setq TeX-auto-save t)
(setq TeX-parse-self t)
(setq-default TeX-master nil)

(defvar latex-imenu-indent-string "."
  "*String to add repeated in front of nested sectional units for Imenu.
An alternative value is \" . \", if you use a font with a narrow period.")

(defun latex-imenu-create-index ()
  "Generates an alist for imenu from a LaTeX buffer."
  (let (i0 menu case-fold-search)
    (save-excursion
      ;; Find the top-most level in this file but don't allow it to be
      ;; any deeper than "section" (which is top-level in an article).
      (goto-char (point-min))
      (if (search-forward-regexp "\\\\part\\*?[ \t]*{" nil t)
          (setq i0 0)
        (if (search-forward-regexp "\\\\chapter\\*?[ \t]*{" nil t)
            (setq i0 1)
          (setq i0 2)))

      ;; Look for chapters and sections.
      (goto-char (point-min))
      (while (search-forward-regexp
              "\\\\\\(part\\|chapter\\|section\\|subsection\\|\
subsubsection\\|paragraph\\|subparagraph\\)\\*?[ \t]*{" nil t)
        (let ((start (match-beginning 0))
              (here (point))
              (i (cdr (assoc (buffer-substring-no-properties
                              (match-beginning 1)
                              (match-end 1))
                             '(("part" . 0) ("chapter" . 1)
                               ("section" . 2) ("subsection" . 3)
                               ("subsubsection" . 4)
                               ("paragraph" . 5) ("subparagraph" . 6))))))
          (backward-char 1)
          (condition-case err
              (progn
                ;; Using sexps allows some use of matching {...} inside
                ;; titles.
                (forward-sexp 1)
                (setq menu
                      (cons (cons (concat (apply 'concat
                                                 (make-list
                                                  (max 0 (- i i0))
                                                  latex-imenu-indent-string))
                                          (buffer-substring-no-properties
                                           here (1- (point))))
                                  start)
                            menu))
                )
            (error nil))))
      
      ;; Look for included material.
      (goto-char (point-min))
      (while (search-forward-regexp
              "\\\\\\(include\\|input\\|verbatiminput\\|bibliography\\)\
[ \t]*{\\([^}\n]+\\)}"
              nil t)
	(setq menu
	      (cons (cons (concat "<<" (buffer-substring-no-properties
					(match-beginning 2)
					(match-end 2))
				  (if (= (char-after (match-beginning 1)) ?b)
				      ".bbl"
				    ".tex"))
			  (match-beginning 0))
		    menu)))

      ;; Look for \frontmatter, \mainmatter, \backmatter, and \appendix.
      (goto-char (point-min))
      (while (search-forward-regexp
	      "\\\\\\(frontmatter\\|mainmatter\\|backmatter\\|appendix\\)\\b"
	      nil t)
	(setq menu
	      (cons (cons "--" (match-beginning 0))
		    menu)))

      ;; Sort in increasing buffer position order.
      (sort menu (function (lambda (a b) (< (cdr a) (cdr b))))))))

                                        ; configure latex hook
(add-hook 'LaTeX-mode-hook 
          '(lambda()
             (and window-system
                  (font-lock-mode 1))
             (auto-fill-mode)
             (setq fill-column 65)
             (setq TeX-open-quote "\"")
             (setq TeX-close-quote "\"")
             (setq tex-open-quote "\"")
             (setq TeX-close-quote "\"")
             (defun TeX-Inserting (sta stb stc)
               (if (= (preceding-char) sta )
                   (insert stb)
                 (progn (insert stc) (backward-char 1))))
             (local-set-key  "{" 'TeX-schweif)
             (local-set-key  "(" 'TeX-rundekl)
             (local-set-key  "[" 'TeX-eckigek)
             (local-set-key  "^" 'TeX-exponen)
             (local-set-key  "_" 'TeX-subscri)
             (local-set-key  "$" 'TeX-dollarm)
             (local-set-key  "\C-b" 'TeX-REVbbox)
             (local-set-key [return] 'reindent-then-newline-and-indent)
             (defun TeX-schweif () 
               (interactive) (TeX-Inserting ?\\ "{"  "{}"))
             (defun TeX-rundekl () 
               (interactive) (TeX-Inserting ?\\ "("  "()"))
             (defun TeX-eckigek () 
               (interactive) (TeX-Inserting ?\\ "["  "[]"))
             (defun TeX-exponen () 
               (interactive) (TeX-Inserting ?\\ "^" "^{}"))
             (defun TeX-subscri () 
               (interactive) (TeX-Inserting ?\\ "_" "_{}"))
             (defun TeX-dollarm () 
               (interactive) (TeX-Inserting ?\\ "$"  "$$"))
             (defun TeX-REVbbox ()
               (interactive) (TeX-Inserting ?\\ "bbox{"  "\\bbox{}"))
             (make-local-variable 'imenu-create-index-function)
             (setq imenu-create-index-function 'latex-imenu-create-index)
             ))

(provide 'auctex-config)

;;; end of auctex-config.el


