;;;
;;; cc-mode-config.el - this is the configuration file for package cc-mode
;;;
;;; Copyright (C) 1998, 1999, 2000, 2001 by Janek Schwarz
;;;
;;; Written by: Janek Schwarz <schwarz@informatik.hu-berlin.de>
;;;
;;; Time-stamp: <2001-10-30 14:34:59 schwarz>
;;;

(require 'config-general)

(fmakunbound 'c-mode)
(fmakunbound 'java-mode)
;(makunbound 'c-mode-map);
(fmakunbound 'c++-mode)
(makunbound 'c++-mode-map);
;(makunbound 'c-style-alist);

(autoload 'c++-mode "cc-mode" "C++ Editing Mode" t)
(autoload 'c-mode   "cc-mode" "C Editing Mode" t)
(autoload 'objc-mode "cc-mode" "Objective-C Editing Mode" t)
(autoload 'java-mode "cc-mode" "Java Editing Mode" t)
(autoload 'idl-mode "cc-mode" "CORBA-IDL Editing Mode")

(setq auto-mode-alist
      (append '(("\\.C$"    . c++-mode)
                ("\\.cc$"   . c++-mode)
                ("\\.hh$"   . c++-mode)
                ("\\.c$"    . c-mode)
                ("\\.h$"    . c++-mode)
                ("\\.m$"    . objc-mode)
                ("\\.java$" . java-mode)
                ("\\.jad$"  . java-mode)
                ("\\.idl$"  . idl-mode)
                ) auto-mode-alist))

                                        ; indentation for C/C++/ObjC code
(defconst my-c-style
  '((c-tab-always-indent           . t)
    (c-hanging-braces-alist        . (brace-list-open))
    (c-basic-offset . 2)
    (c-comment-only-line-offset . (0 . 0))
    (c-cleanup-list                . (scope-operator
                                      empty-defun-braces
                                      defun-close-semi
                                      list-close-comma))
    (c-offsets-alist . ((statement-block-intro . +)
                        (knr-argdecl-intro . 5)
                        (substatement-open . +)
                        (label . 0)
                        (statement-case-open . +)
                        (statement-cont . +)
                        (arglist-close . c-lineup-arglist)))
    (c-special-indent-hook . c-gnu-impose-minimum))
  "Janeks C/C++ Programming Style")

(defconst my-java-style
  '((c-tab-always-indent           . t)
    (c-hanging-braces-alist        . (brace-list-open))
    (c-basic-offset . 2)
    (c-comment-only-line-offset . (0 . 0))
    (c-cleanup-list                . (scope-operator
                                      empty-defun-braces
                                      defun-close-semi
                                      list-close-comma))
    (c-offsets-alist . ((statement-block-intro . +)
                        (inclass . 0)
                        (inline-open . 0)
                        (topmost-intro-cont . +)
                        (knr-argdecl-intro . 5)
                        (substatement-open . +)
                        (label . 0)
                        (statement-case-open . +)
                        (statement-cont . +)
                        (arglist-close . c-lineup-arglist)))
    (c-special-indent-hook . c-gnu-impose-minimum))
  "Janeks Java Programming Style")


; common CC-mode hook
(defun my-c-mode-common-hook ()
  ;; add my personal style and set it for the current buffer
  (c-add-style "JANEK-C" my-c-style t)
  (setq tab-width 8
        indent-tabs-mode nil);; make sure to use spaces instead of tabs
  (c-toggle-auto-hungry-state 1)

  (c-setup-filladapt)
  (filladapt-mode 1)
  (define-key c-mode-map "\C-m" 'c-context-line-break)
  (define-key c-mode-base-map "\C-c\C-a"  'align))

(add-hook 'c-mode-common-hook 'my-c-mode-common-hook)

                                        ;c-mode
(add-hook 'c-mode-hook
	  '(lambda ()
	     (and window-system
		  (progn
		    (font-lock-mode t)
		    (setq c-font-lock-keywords c-font-lock-keywords-2)))))

                                        ; ObjC mode
(add-hook 'objc-mode-hook
	  '(lambda ()
	     (c-toggle-auto-hungry-state t)       ;Auto-Newline
             ))

                                        ;C++-Mode
(add-hook 'c++-mode-hook
	  '(lambda ()
	     (and window-system
		  (progn
		    (font-lock-mode t)))))

(add-hook 'java-mode-hook
          '(lambda ()
             (c-add-style "JANEK-J" my-java-style t)
             (setq case-fold-search t)
             (camelCase-mode 1)
             ))

(provide 'cc-mode-config)

;;; end of cc-mode-config.el
