;;;
;;; config-general.el - this is the general configuration file for my emacs
;;;
;;; Copyright (C) 1998, 1999, 2000, 2001, 2002 by Janek Schwarz
;;;
;;; Written by: Janek Schwarz <schwarz@informatik.hu-berlin.de>
;;;
;;; Time-stamp: <2002-01-20 17:06:37 schwarz>
;;;

(standard-display-european 1)           ; deutsche Sonderzeichen anzeigen
(set-input-mode (car (current-input-mode)) ; deutsche Sonderzeichen eingeben
                (nth 1 (current-input-mode))
                0)

(require 'keys)

(require 'filladapt)
(setq-default filladapt-mode t)
(add-hook 'text-mode-hook 'turn-on-filladapt-mode)

(setq scroll-step 5)                    ; bei Scrollen ueber Bildrand
(line-number-mode 1)                    ; show line number
(column-number-mode 1)                  ; show column number
(setq font-lock-maximum-decoration t)   ; as colorful as possible
(setq visible-bell 1)                   ; don't beep
(setq default-major-mode 'auto-fill-mode) ; mode for unknown files
(setq default-fill-column 78)           ; text width
(setq-default indent-tabs-mode nil)	; insert spaces instead of tabs
(setq-default tab-width 2)
(setq transient-mark-mode nil)          ; don't mark as in windows
;(setq find-file-visit-truename t)
;(setq shell-file-name "bash.exe")
(add-hook 'write-file-hooks 'time-stamp); update timestamp
(add-hook 'write-file-hooks 'copyright-update) ; update copyright
(fset 'yes-or-no-p 'y-or-n-p)           ; replace y-e-s by y

(autoload 'emacs-lock "emacs-lock"      ; lock; prevents closing emacs
  "lock emacs" t)                       ; accidently

                                        ; Inspect
(autoload 'load-inspect-mode "inspect"  ; Betrachten von Variablen beim
  "Inspector addition" t)               ; Debuggen in anderem Fenster, C++

                                        ; Postscript-Modus
(autoload 'postscript-mode "postscript.el" 
  "Mode for editing postscript files" t) 

                                        ; FVWM-Modus
(autoload 'fvwm-mode "fvwm-mode"
  "Mode for editing fvwm config files" t)
                                    
(setq auto-mode-alist
      (append '(("\\.fvwm$" . fvwm-mode)
  		(".fvwmrc$" . fvwm-mode)
		(".bowmanrc" . fvwm-mode)
                (".steprc" . fvwm-mode)
                ("\\.c?ps$".postscript-mode))
	      auto-mode-alist))

(if window-system
    (progn
      (require 'mic-paren)
      (setq paren-face 'bold)
      (setq paren-sexp-mode t)))

(resize-minibuffer-mode t)

; for some reason, emacs on OS/2 doesn't like crypt++.
;(if (not (eq window-system 'pm))
;    (require 'crypt++))

;there's a similar function built into emacs 21
;(load "backup-dir")
;(setq backup-directory (expand-file-name "~/.emacs-auto"))

;;; OS/2 specific stuff
(if (eq window-system 'pm)
    (progn
      (setq case-fold-file-names nil)
      ;;; shell-mode is broken in emacs/2 20.3.1.3
      (setq auto-mode-alist
            (append '(("\\.[ck]?sh\\'\\|\\.shar\\'\\|/\\.z?profile\\'" . auto-fill-mode)
                      ("\\(/\\|\\`\\)\\.\\(bash_profile\\|z?login\\|bash_login\\|z?logout\\)\\'" . auto-fill-mode)
                      ("\\(/\\|\\`\\)\\.\\(bash_logout\\|[kz]shrc\\|bashrc\\|t?cshrc\\|esrc\\)\\'" . auto-fill-mode)
                      ("\\(/\\|\\`\\)\\.\\([kz]shenv\\|xinitrc\\|startxrc\\|xsession\\)\\'" . sh-mode)
                      ) auto-mode-alist))))
 
(provide 'config-general)

;;; end of config-general.el
