;;;
;;; ecb-config.el - this is the configuration file for package 
;;;
;;; Copyright (C) 2001, 2002 by Janek Schwarz
;;;
;;; Written by: Janek Schwarz <schwarz@informatik.hu-berlin.de>
;;;
;;; Time-stamp: <2002-11-04 09:32:17 schwarz>
;;;

(require 'config-general)

(require 'ecb)
(require 'eshell)

(setq truncate-partial-width-windows nil) ; Prevent ECB from
                                        ; truncating the edit windows.


(defun jsz-ecb-switch-to-compilation-buffer()
  "Switches to compilation buffer in other window"
  (interactive)
  (jsz-ecb-switch-to-buffer "*compilation*"))

(defun jsz-ecb-switch-to-buffer(buffer-name)
  "Switches to buffer named 'buffer-name'"
  (if (get-buffer buffer-name)
      (switch-to-buffer-other-window buffer-name)
    (message "Kein %s buffer vorhanden" buffer-name)))

(defun jsz-ecb-switch-to-eshell-buffer (&optional arg)
  "Create an interactive Eshell buffer.
The buffer used for Eshell sessions is determined by the value of
`eshell-buffer-name'.  If there is already an Eshell session active in
that buffer, Emacs will simply switch to it.  Otherwise, a new session
will begin.  A new session is always created if the the prefix
argument ARG is specified.  Returns the buffer selected (or created)."
  (interactive "P")
  (assert eshell-buffer-name)
  (let ((buf (if arg
		 (generate-new-buffer eshell-buffer-name)
	       (get-buffer-create eshell-buffer-name))))
    ;; Simply calling `pop-to-buffer' will not mimic the way that
    ;; shell-mode buffers appear, since they always reuse the same
    ;; window that that command was invoked from.  To achieve this,
    ;; it's necessary to add `eshell-buffer-name' to the variable
    ;; `same-window-buffer-names', which is done when Eshell is loaded
    (assert (and buf (buffer-live-p buf)))
    (switch-to-buffer-other-window buf)
    (unless (fboundp 'eshell-mode)
      (error "`eshell-auto' must be loaded before Eshell can be used"))
    (unless (eq major-mode 'eshell-mode)
      (eshell-mode))
    (assert (eq major-mode 'eshell-mode))
    buf))

(provide 'ecb-config)

;;; end of ecb-config.el
